/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface mozIDOMWindow;

/**
 * Start and stop the remote service (xremote/phremote), and register
 * windows with the service for backwards compatibility with old xremote
 * clients.
 *
 * @status FLUID This interface is not frozen and is not intended for embedders
 *               who want a frozen API. If you are an embedder and need this
 *               functionality, contact Benjamin Smedberg about the possibility
 *               of freezing the functionality you need.
 */

[scriptable, uuid(bf23f1c3-7012-42dd-b0bb-a84060ccc52e)]
interface nsIRemoteService : nsISupports
{
  /**
   * Start the remote service. This should not be done until app startup
   * appears to have been successful.
   *
   * @param appName     (Required) Sets a window property identifying the
   *                    application.
   * @param profileName (May be null) Sets a window property identifying the
   *                    profile name.
   */
  void startup(in string appName, in string profileName);

  /**
   * Register a XUL window with the xremote service. The window will be
   * configured to accept incoming remote requests. If this method is called
   * before startup(), the registration will happen once startup() is called.
   */
  void registerWindow(in mozIDOMWindow aWindow);

  /**
   * Stop the remote service from accepting additional requests.
   */
  void shutdown();
};
