#define MOZ_SOURCE_STAMP 2d931a5eaf8aeee925eca2aea42582a1fb9793c8
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/2d931a5eaf8aeee925eca2aea42582a1fb9793c8
