/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface mozIDOMWindowProxy;

/*
	The nsPICommandUpdater interface is used by modules that implement
	commands, to tell the command manager that commands need updating.
	This is a private interface; embedders should not use it.
	
	Command-implementing modules should get one of these by a QI
	from an nsICommandManager.
*/

[scriptable, uuid(35e474ae-8016-4c34-9644-edc11f8b0ce1)]
interface nsPICommandUpdater : nsISupports
{

  /*
   * Init the command updater, passing an nsIDOMWindow which 
   * is the window that the command updater lives on.
   * 
   */
  void    init(in mozIDOMWindowProxy aWindow);
   
  /*
   * Notify the command manager that the status of a command
   * changed. It may have changed from enabled to disabled,
   * or vice versa, or become toggled etc.
   */
	void		commandStatusChanged(in string aCommandName);
	
};


